import gui.OgreMailTableModel;
import gui.OgrePrincipal;
import gui.OgreProgressBar;
import gui.OgreSplashScreen;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;
import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JSplitPane;
import javax.swing.UIManager;
import base.MailProperties;
import base.MainOgre;

/******************************************************************************
 * 																			  *
 *  Copyright 2004 Jordi Oliveras & Marcos Valerio.							  *
 * 																			  *
 *  This file is part of Ogre.												  *
 *																			  *
 *  Ogre is free software; you can redistribute it and/or modify			  *
 *  it under the terms of the GNU General Public License as published by	  *
 *  the Free Software Foundation; either version 2 of the License, or		  *
 *  (at your option) any later version.										  *
 *																			  *
 *  Ogre is distributed in the hope that it will be useful,					  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of			  *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the			  *
 *  GNU General Public License for more details.							  *
 *                       													  *
 *  You should have received a copy of the GNU General Public License         *
 *  along with Ogre; if not, write to the Free Software                       *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 * 																			  *
 ******************************************************************************
 * 																			  *
 * Ogre - Smil del 'main' de una aplicacin.                                 *
 * 																			  *
 ******************************************************************************/

public final class Ogre {

	static JFrame frame;

	public static void main(String[] args) throws IOException {
		
		try {
		
			OgreSplashScreen oss = new OgreSplashScreen(); // Hola splashscreen.
			
			// Activacin DEBUG. Se hace directamente desde MainOgre.

			// Activacin log errores (switch entre modo consola/modo log).
			//MainOgre.activar_log_errores();
			
			// Gestion L&F (multi-arquitectura).
			try {
				UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
			} catch (Exception e){
				JOptionPane.showMessageDialog(null, e.getMessage()+MainOgre.mensajeErrorEstandard, MailProperties.textos.getString("error"), JOptionPane.ERROR_MESSAGE);
				e.getMessage();
				e.printStackTrace();
			}
			
			// Establecimiento de configuraciones y archivos/directorios.
			MainOgre.establecer_config();
			MainOgre.establecer_mailbox_y_agenda();
			
			// Crear el top-level container y aadirle los contenidos.
			frame = new JFrame(MailProperties.xmailer);
			
			// Creacin del modelo de la tabla.
			OgreMailTableModel omtm = new OgreMailTableModel();
			
			// Creacin de toda la ventana inicial.
			JSplitPane jsplitpane1 = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
			OgreProgressBar opb = new OgreProgressBar();
			OgrePrincipal app = new OgrePrincipal(frame, jsplitpane1, omtm);
			
			frame.getContentPane().add(jsplitpane1, BorderLayout.CENTER);
			frame.getContentPane().add(opb, BorderLayout.SOUTH);
			
			/* 
		 	* Desde que NO usamos un contenedor de alto nivel para hacer de content pane
		 	* (como un JPanel), si queremos poder tocar los bordes, debemos hacer un cast
		 	* (a un JComponent) de lo que nos devuelve el frame al pedirle su content pane.
		 	* De esta forma ya podemos operar con los bordes. */
			JComponent contentPane = (JComponent) frame.getContentPane();
			contentPane.setBorder(BorderFactory.createEmptyBorder(
			2, //top
			10, //left
			5, //bottom
			10) //right
			);
			
			// Terminamos de construir el frame y lo mostramos.
			frame.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					// Cerramos la ventana.
					frame.setVisible(false);
					frame.dispose();
					System.exit(0);
				}
			});
			
			// Posicionamiento ventana principal de la aplicacin.
			Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
			Dimension frameSize = frame.getPreferredSize();
			frame.setLocation(screenSize.width/2 - (frameSize.width/2), screenSize.height/2 - (frameSize.height/2));
			
			// Adios splashscreen.
			oss.hide();
			oss.dispose();
			
			frame.pack();
			frame.setVisible(true);
			
		} catch(Exception e){
			JOptionPane.showMessageDialog(null, e.getMessage()+MainOgre.mensajeErrorEstandard, MailProperties.textos.getString("error"), JOptionPane.ERROR_MESSAGE);
			e.getMessage();
			e.printStackTrace();
		}
			
	}
}
