package base;
    
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import javax.swing.JOptionPane;

/******************************************************************************
 * 																			  *
 *  Copyright 2004 Jordi Oliveras & Marcos Valerio. 						  *
 * 																			  *
 *  This file is part of Ogre.												  *
 *																			  *
 *  Ogre is free software; you can redistribute it and/or modify			  *
 *  it under the terms of the GNU General Public License as published by	  *
 *  the Free Software Foundation; either version 2 of the License, or		  *
 *  (at your option) any later version.										  *
 *																			  *
 *  Ogre is distributed in the hope that it will be useful,					  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of			  *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the			  *
 *  GNU General Public License for more details.							  *
 *                       													  *
 *  You should have received a copy of the GNU General Public License         *
 *  along with Ogre; if not, write to the Free Software                       *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 * 																			  *
 ******************************************************************************
 * 																			  *
 * Agenda - Clase que representa la agenda del cliente de correo, para operar *
 * con los contactos en ella guardados.										  *
 * 																			  *
 ******************************************************************************/

public final class Agenda {

	MailProperties mailProperties;

	public Agenda(MailProperties mailProperties){
		this.mailProperties = mailProperties;
	}

	/* ---Mtodo cargar_agenda--- */
	/* Si no existe, crea el fichero que sera la Agenda. Si existe y almacena
	 * informacin, la carga como Agenda. */
	public void cargar_agenda(ArrayList agendaMemoria){
		
		try {
			
			File file = new File(MailProperties.path_agenda+"/Contactos");
			
			// Si no existe el fichero de contactos, lo creamos
			if(!file.exists()){
				
				FileOutputStream fos = new FileOutputStream(MailProperties.path_agenda+"/Contactos");
				fos.close();
				
			// Si existe, y su tamao es >0, lo cargamos en memoria
			} else if(file.length()>0){
			
				FileInputStream fis = new FileInputStream(MailProperties.path_agenda+"/Contactos");
				ObjectInputStream p = new ObjectInputStream(fis);
			
				try {
					
					// Vamos cargando en memoria todos los contactos
					while(true){
						Contacto unContacto = (Contacto) p.readObject();
						agendaMemoria.add(unContacto);
					}
				} catch(Exception e) {
					System.err.println("Contactos cargados.");
				}
				
				fis.close();
			}
						
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, e.getMessage(), "Error en cargar_agenda()", JOptionPane.ERROR_MESSAGE);
			//System.out.println("Error en cargar_agenda(): "+e.getMessage());
			e.printStackTrace();
		}
	}
	
	/* ---Mtodo restaurar_agenda--- */
	/* Restaura un estado anterior de la Agenda. */
	public ArrayList restaurar_agenda(ListRestorer agendaRest, ArrayList agendaMemoria){ // Esta operacin no debera ser backupped tambin?
		agendaMemoria = agendaRest.restore(agendaMemoria);
		return agendaMemoria;
	}
	
	/* ---Mtodo ordenar_agenda--- */
	/* Ordena alfabticamente la Agenda. */
	public void ordenar_agenda(ArrayList agendaMemoria){
		Collections.sort(agendaMemoria, new ListComparator()); // Implementacin externa a Contacto
		//Collections.reverse(lista); --> funciona
	}

	/* ---Mtodo actualizar_agenda--- */
	/* Guarda en disco la Agenda ms nueva. */
	public void actualizar_agenda(ArrayList agendaMemoria){
		try {
	
			File file = new File(MailProperties.path_agenda+"/Contactos");
			// Si el fichero existe, lo borramos
			if(file.exists()){
				file.delete();
			}
			
			// Si no hay contacto alguno, no guardamos nada
			if(!agendaMemoria.isEmpty()){
	
				FileOutputStream ostream = new FileOutputStream(MailProperties.path_agenda+"/Contactos");
				ObjectOutputStream p = new ObjectOutputStream(ostream);
					
				Iterator iterator = agendaMemoria.iterator();
				while(iterator.hasNext()){
					Contacto unContacto = (Contacto) iterator.next();
					p.writeObject(unContacto);
				}
			
				ostream.close();
			}
	
		} catch(Exception e){
			JOptionPane.showMessageDialog(null, e.getMessage(), "Error en actualizar_agenda()", JOptionPane.ERROR_MESSAGE);
			//System.out.println("Error en actualizar_agenda(): "+e.getMessage());
			e.printStackTrace();
		}
	}
	
	/* ---Mtodo destruir_agenda--- */
	/* Destruye la Agenda, tanto la situada en memoria como fsicamente en disco. */
	public void destruir_agenda(){
		File file = new File(MailProperties.path_agenda+"/Contactos");
		// Si el fichero existe, lo borramos
		if(file.exists()){
			file.delete();
			if(MainOgre.verboso) System.out.println("Agenda borrada!");
		} else {
			if(MainOgre.verboso) System.out.println("Agenda NO borrada! No se encuentra el fichero!");
		}
	}
}
