package base;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

/******************************************************************************
 * 																			  *
 *  Copyright 2004 Jordi Oliveras & Marcos Valerio. 						  *
 * 																			  *
 *  This file is part of Ogre.												  *
 *																			  *
 *  Ogre is free software; you can redistribute it and/or modify			  *
 *  it under the terms of the GNU General Public License as published by	  *
 *  the Free Software Foundation; either version 2 of the License, or		  *
 *  (at your option) any later version.										  *
 *																			  *
 *  Ogre is distributed in the hope that it will be useful,					  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of			  *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the			  *
 *  GNU General Public License for more details.							  *
 *                       													  *
 *  You should have received a copy of the GNU General Public License         *
 *  along with Ogre; if not, write to the Free Software                       *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 * 																			  *
 ******************************************************************************
 * 																			  *
 * CuentaMail - Clase que representa una cuenta de correo, sus atributos, y   *
 * las operaciones que se pueden realizar sobre los mismos.					  *
 * 																			  *
 ******************************************************************************/

public final class CuentaMail implements Serializable {

	static String linea;
	static final long serialVersionUID = 1; // N para serializar objetos.
	public String nombreCuenta, hostSmtp, hostPop, nombreUsuario, contrasea, email;
	
	/* ---Mtodo crear_cuenta--- */
	/* Crea una cuenta de mail, guardndola en un fichero con el nombre de la
	 * Cuenta. Esto se hace serializando un objeto de tipo CuentaMail. */
	public boolean crear_cuenta() throws IOException{ 
		
		boolean resultado;
		
		if(existe_fichero(this.nombreCuenta)){
			// Existe la cuenta, no se puede sobreescribir.
			if(MainOgre.verboso) System.out.println("Ya existe la cuenta, no se puede sobreescribir!"); // DEBUG
			resultado = false; 
		} else {
			// Lo guardamos a disco.
			File file = new File(MailProperties.path_ogre+"."+this.nombreCuenta+"_config");
			FileOutputStream fos = new FileOutputStream(MailProperties.path_ogre+"."+this.nombreCuenta+"_config");
			ObjectOutputStream p = new ObjectOutputStream(fos);
			p.writeObject(this);
			fos.close();
			resultado = true;
			if(MainOgre.verboso) System.out.println("Cuenta creada."); // DEBUG
		}
			
		crear_estructura_mailbox(this.nombreCuenta);
		
		return resultado;
	}

	/* ---Mtodo crear_cuenta_estatica--- */
//	public void crear_cuenta_estatica() throws IOException{
//		
//		if(existe_fichero(this.nombreCuenta)){
//			// Existe la cuenta, no se puede sobreescribir.
//			if(MainOgre.verboso) System.out.println("Ya existe la cuenta, no se puede sobreescribir!"); // DEBUG
//		} else {
//			// Lo guardamos a disco.
//			File file = new File(MailProperties.path_ogre+"."+this.nombreCuenta+"_config");
//			FileOutputStream fos = new FileOutputStream(MailProperties.path_ogre+"."+this.nombreCuenta+"_config");
//			ObjectOutputStream p = new ObjectOutputStream(fos);
//			p.writeObject(this);
//			fos.close();
//			
//			crear_estructura_mailbox(this.nombreCuenta);
//			
//			if(MainOgre.verboso) System.out.println("Cuenta creada."); // DEBUG
//		}
//	}

	/* ---Mtodo crear_estructura_mailbox--- */
	/* Crea una estructura de tipo mailbox para cada cuenta que se cree. */
	private void crear_estructura_mailbox(String nombreCuenta){
		
		String estructura_dirs[] = {"inbox", "outbox", "queued", "draft", "trash"};
		/* Entrantes, Salientes, Encolados(p.ej.no pudieron enviarse), Borradores y Basura */
		
		// Comprobacion estructura mailbox.
		File file = new File(MailProperties.path_ogre+nombreCuenta);
		if(!file.exists()){
			boolean estado = file.mkdir();
			if(!estado){
				System.out.println("No se cre "+nombreCuenta+"; algo fue mal!"); // DEBUG
				System.out.println("Cuestin de permisos?Path incorecto?"); // DEBUG
				System.out.println("Saliendo!!!"); // DEBUG
				System.exit(0);
			}
				
			int i=0;
			while((i<estructura_dirs.length)&&(estado==true)){
				file = new File(MailProperties.path_ogre+nombreCuenta+"/"+estructura_dirs[i]); // esto PUEDE FALLAR en SO's no-*nix!!!
				estado = file.mkdir();
				i++;
			}
			if(estado==false){
				System.out.println("No se cre el mailbox; algo fue mal!"); // DEBUG
				System.out.println("Cuestin de permisos?Path incorecto?"); // DEBUG
				System.out.println("Saliendo!!!"); // DEBUG
				System.exit(0);
			}
		}
	}

	/* ---Mtodo modificar_cuenta--- */
	/* Modifica la cuenta elegida. */
	public void modificar_cuenta() throws IOException{
		
		// Borramos el archivo de la cuenta a editar, y creamos uno nuevo con
		// la nueva informacin de la Cuenta.
		File file = new File(MailProperties.path_ogre+"."+nombreCuenta+"_config");
		file.delete();
			
		file = new File(MailProperties.path_ogre+"."+nombreCuenta+"_config");
		FileOutputStream fos = new FileOutputStream(MailProperties.path_ogre+"."+nombreCuenta+"_config");
		ObjectOutputStream p = new ObjectOutputStream(fos);
		p.writeObject(this);
		fos.close();
		
		if(MainOgre.verboso) System.out.println("Cuenta modificada."); // DEBUG
	}

	/* ---Mtodo eliminar_cuenta--- */
	/* Elimina una cuenta. */
	public void eliminar_cuenta(String nombreCuenta){
		
		// Se comprueba que exista el fichero que se quiere borrar.
		if(!existe_fichero(nombreCuenta)){
			if(MainOgre.verboso) System.out.println("No existe la cuenta, no se puede borrar!"); // DEBUG
		} else {
			File file = new File(MailProperties.path_ogre+"."+nombreCuenta+"_config");
			if(file.delete()){
				if(MailProperties.jck3==true)
					eliminar_recursivo(MailProperties.path_ogre+nombreCuenta);
				if(MainOgre.verboso) System.out.println("Cuenta borrada!"); // DEBUG
			}else {
				if(MainOgre.verboso) System.out.println("Cuenta NO borrada!"); // DEBUG
			}
		}
	}

	/* ---Mtodo eliminar_recursivo--- */
	/* Elimina recursivamente un directorio y todo lo que ste contenga. */
	private void eliminar_recursivo(String nombreDirPadre){
	
		File file = new File(nombreDirPadre);
		if(file.exists()){
			
			String absolutePath = file.getAbsolutePath();
			if(MainOgre.verboso) System.out.println("absolutePath: "+absolutePath); // DEBUG
			String arrayStrings[] = file.list();
				
			for(int i=0; i<arrayStrings.length; i++){
				if(MainOgre.verboso) System.out.println("arrayStrings["+i+"]: "+arrayStrings[i]); // DEBUG
				file = new File(absolutePath+"/"+arrayStrings[i]);
				if(file.isFile())
					file.delete();
				else if(file.isDirectory()){
					eliminar_recursivo(file.getAbsolutePath());
					file.delete(); // Borramos el directorio una vez esta vacio.
				}
			}
			file = new File(nombreDirPadre);
			file.delete(); // Borramos el directorio padre.
		}
	}

	/* ---Mtodo seleccionar_cuenta--- */
	/* Selecciona la cuenta actual con la que se desea operar. */
	public void seleccionar_cuenta(String unNombreCuenta) throws IOException, ClassNotFoundException{
			
		// Se comprueba que exista ese fichero (no queremos sobreescribirlo).
		if(!existe_fichero(unNombreCuenta)){
			// Existe la cuenta, no se puede sobreescribir.
			if(MainOgre.verboso) System.out.println("No existe la cuenta, no se puede seleccionar!");  // DEBUG
		} else {
			if(MainOgre.verboso) System.out.println("nombreCuenta en CuentaMail.seleccionar_cuenta(): "+unNombreCuenta);  // DEBUG
			// Existe, por tanto leemos la cuenta selecionada de disco.
			if(MainOgre.verboso) System.out.println("Path para abrir la cuenta: "+MailProperties.path_ogre+"."+unNombreCuenta+"_config"); // DEBUG
			File file = new File(MailProperties.path_ogre+"."+unNombreCuenta+"_config");
			FileInputStream fis = new FileInputStream(MailProperties.path_ogre+"."+unNombreCuenta+"_config");
			ObjectInputStream p = new ObjectInputStream(fis);
			BufferedInputStream bis = new BufferedInputStream(p); // Con buffer.
			CuentaMail cuentaMail = (CuentaMail) p.readObject();
			bis.close();
			fis.close();
			
			// Y cargamos la configuracin de la cuenta seleccionada.
			nombreCuenta = cuentaMail.nombreCuenta;
			hostSmtp = cuentaMail.hostSmtp;
			hostPop = cuentaMail.hostPop;
			nombreUsuario = cuentaMail.nombreUsuario;
			contrasea = cuentaMail.contrasea;
			email = cuentaMail.email;
			
			if(MainOgre.verboso) System.out.println("Cuenta seleccionada.");
		}
	}
	
	/* ---Mtodo descubrir_cuentas--- */
	/* Encuentra las cuentas disponibles para un usuario dado. */
	public String[] descubrir_cuentas(){
		
		int i = 0, j = 0;
		
		File file = new File(MailProperties.path_ogre);
		String arrayStrings[] = file.list();
		
		// Contamos las cuentas disponibles.
		while(i < arrayStrings.length){
			if((arrayStrings[i].startsWith("."))&&(arrayStrings[i].endsWith("_config"))){
				j++;
			}
			i++;
		}
		
		i = 0;
		String cuentas[] = new String[j];
		j = 0;
		
		// Mientras haya archivos, mira que no sean cuentas.
		while(i < arrayStrings.length){
			if((arrayStrings[i].startsWith("."))&&(arrayStrings[i].endsWith("_config"))){
				if(MainOgre.verboso) System.out.println(arrayStrings[i]);
				cuentas[j] = arrayStrings[i]; // Si lo son las guarda en un array.
				j++;
			}
			i++;
		}
		
		return cuentas;
	}
	
	/* ---Mtodo mostrar_cuenta--- */
	/* Muestra el contenido de la cuenta actual. */
	public void mostrar_cuenta(){
		if(MainOgre.verboso){
			System.out.println("....................................");
			System.out.println("Nombre de la Cuenta: "+nombreCuenta);
			System.out.println("Host smtp: "+hostSmtp);
			System.out.println("Host pop: "+hostPop);
			System.out.println("Nombre de Usuario: "+nombreUsuario);
			System.out.println("Contrasea: "+contrasea);
			System.out.println("E-Mail: "+email);
			System.out.println("....................................");
		}
	}
	
	/* ---Mtodo existe_fichero--- */
	/* Comprueba la existencia o ausencia de un nombre de fichero, que se corresponde
	 * con el nombre de una Cuenta. */
	public boolean existe_fichero(String nombre){
		
		boolean existe;
		
		if(MainOgre.verboso) System.out.println("Ruta a la cuenta: "+MailProperties.path_ogre+"."+nombre+"_config");
		File file = new File(MailProperties.path_ogre+"."+nombre+"_config");
		if(file.exists()){
			existe = true; // Existe la cuenta, no se puede sobreescribir.
		} else {
			existe = false; // No existe.
		}
		
		return existe;
	}

}
