package base;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.Serializable;
import java.util.Locale;
import java.util.Properties;
import java.util.ResourceBundle;

/******************************************************************************
 * 																			  *
 *  Copyright 2004 Jordi Oliveras & Marcos Valerio. 						  *
 * 																			  *
 *  This file is part of Ogre.												  *
 *																			  *
 *  Ogre is free software; you can redistribute it and/or modify			  *
 *  it under the terms of the GNU General Public License as published by	  *
 *  the Free Software Foundation; either version 2 of the License, or		  *
 *  (at your option) any later version.										  *
 *																			  *
 *  Ogre is distributed in the hope that it will be useful,					  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of			  *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the			  *
 *  GNU General Public License for more details.							  *
 *                       													  *
 *  You should have received a copy of the GNU General Public License         *
 *  along with Ogre; if not, write to the Free Software                       *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 * 																			  *
 ****************************************************************************** 
 * 																			  *
 * MailProperties - Clase encargada de establecer ciertas propiedades bsicas *
 * para el correcto funcionamiento de la aplicacin. Actualmente son pocas,   *
 * pero se prev que en un futuro sern ms extensas cuando se aada a la     *
 * aplicacin la posibilidad de personalizacin del cliente.				  *
 *																			  *
 ******************************************************************************/

public final class MailProperties implements Serializable {

	public static String path_ogre, path_agenda, xmailer;
	static String arrayCamposFijos[] = {"path_ogre", "path_agenda", "xmailer"};
	static String arrayValoresFijosDefault[] = {System.getProperty("user.home")+"/.Ogre/",
									System.getProperty("user.home")+"/.Ogre"+"/agenda",
									"Ogre 0.2"};
	public static boolean jck1, jck2, jck3, jck4, jck5, jck6;
	static String arrayCamposBoolean[] = {"jck1", "jck2", "jck3", "jck4", "jck5", "jck6"};
	static boolean arrayValoresBooleanDefault[] = {true, true, true, false, false, false};
	
	public static String jcb1, jcb2, lang;
	static String arrayCamposString[] = {"jcb1", "jcb2", "lang"};
	//static String arrayValoresStringDefault[] = {"30 minutos", System.getProperty("user.language")};
	static String arrayValoresStringDefault[] = {"", "", System.getProperty("user.language")};
	
	public static Locale localeActual;
	public static ResourceBundle textos;
	
	
	/* ---Mtodo escribir_properties_default--- */
	/* Escribe en el fichero de configuracin de la aplicacin las propiedades
	 * por defecto de sta. */
	public static void escribir_properties_default() throws IOException{
		
		Properties properties = new Properties();
		FileOutputStream fos = new FileOutputStream(System.getProperty("user.home")+"/.Ogre.properties");
			
		// Establecemos todas las properties por defecto.
		for(int i=0; i<arrayCamposFijos.length; i++){
			if(MainOgre.verboso) System.out.println("arrayCamposFijos[i]: "+arrayCamposFijos[i]+", arrayValoresFijosDefault[i]: "+arrayValoresFijosDefault[i]);
			properties.setProperty(arrayCamposFijos[i], arrayValoresFijosDefault[i]);
		}
			
		for(int i=0; i<arrayCamposBoolean.length; i++){
			if(MainOgre.verboso) System.out.println("arrayCamposBoolean[i]: "+arrayCamposBoolean[i]+", arrayValoresBooleanDefault[i]: "+(new Boolean(arrayValoresBooleanDefault[i])).toString());
			properties.setProperty(arrayCamposBoolean[i], (new Boolean(arrayValoresBooleanDefault[i])).toString());
		}
		
		for(int i=0; i<arrayCamposString.length; i++){
			if(MainOgre.verboso) System.out.println("arrayCamposString[i]: "+arrayCamposString[i]+", arrayValoresStringDefault[i]: "+(new Boolean(arrayValoresStringDefault[i])).toString());
			if(i!=2)
				properties.setProperty(arrayCamposString[i], (new Boolean(arrayValoresStringDefault[i])).toString());
			else
				properties.setProperty(arrayCamposString[i], arrayValoresStringDefault[i]);
		}
			
		// Las guardamos en el fichero.
		properties.store(fos, "Ogre Global");
			
		// Cerramos el fichero.
		fos.close();
		
		//System.out.println(System.getProperty("user.language"));
	}

	/* ---Mtodo escribir_properties--- */
	/* Escribe de nuevo las propiedades, presumiblemente porque habrn cambiado. Esto
	 * se da, por ejemplo, al cambiar la configuracin del cliente. */
	public static void escribir_properties() throws IOException{
		
		Properties properties = new Properties();
		FileOutputStream fos = new FileOutputStream(System.getProperty("user.home")+"/.Ogre.properties");
		
		// Establecemos algunas properties por defecto...
		for(int i=0; i<arrayCamposFijos.length; i++)
			properties.setProperty(arrayCamposFijos[i], arrayValoresFijosDefault[i]);
		
		// ...y luego establecemos las nuevas.
		properties.setProperty(arrayCamposBoolean[0], new Boolean(jck1).toString());
		properties.setProperty(arrayCamposBoolean[1], new Boolean(jck2).toString());
		properties.setProperty(arrayCamposBoolean[2], new Boolean(jck3).toString());
		properties.setProperty(arrayCamposBoolean[3], new Boolean(jck4).toString());
		properties.setProperty(arrayCamposBoolean[4], new Boolean(jck5).toString());
		properties.setProperty(arrayCamposBoolean[5], new Boolean(jck6).toString());
		properties.setProperty(arrayCamposString[0], jcb1);
		properties.setProperty(arrayCamposString[1], jcb2);
		properties.setProperty(arrayCamposString[2], lang);
		
		// Las guardamos en el fichero.
		properties.store(fos, "Ogre Global");
			
		// Cerramos el fichero.
		fos.close();
	}

	/* ---Mtodo leer_properties--- */
	/* Lee las propiedades del fichero de configuracin de la aplicacin y las almacena
	 * en variables de clase para que estas puedan ser accedidas por la aplicacion sin
	 * necesidad de leer de nuevo el fichero de configuracion. */
	public static void leer_properties() throws IOException{

		Properties properties = new Properties();
		FileInputStream fis = new FileInputStream(System.getProperty("user.home")+"/.Ogre.properties");
			
		// Carga el fichero de properties.
		properties.load(fis);
			
		// Leemos todas las properties.
		path_ogre = properties.getProperty(arrayCamposFijos[0], "Ogre Global");
		path_agenda = properties.getProperty(arrayCamposFijos[1],"Ogre Global");
		xmailer = properties.getProperty(arrayCamposFijos[2],"Ogre Global");
		jck1 =  new Boolean(properties.getProperty(arrayCamposBoolean[0], "Ogre Global")).booleanValue();
		jck2 =  new Boolean(properties.getProperty(arrayCamposBoolean[1], "Ogre Global")).booleanValue();
		jck3 =  new Boolean(properties.getProperty(arrayCamposBoolean[2], "Ogre Global")).booleanValue();
		jck4 =  new Boolean(properties.getProperty(arrayCamposBoolean[3], "Ogre Global")).booleanValue();
		jck5 =  new Boolean(properties.getProperty(arrayCamposBoolean[4], "Ogre Global")).booleanValue();
		jck6 =  new Boolean(properties.getProperty(arrayCamposBoolean[5], "Ogre Global")).booleanValue();
		jcb1 = properties.getProperty(arrayCamposString[0],"Ogre Global");
		jcb2 = properties.getProperty(arrayCamposString[1],"Ogre Global");
		lang = properties.getProperty(arrayCamposString[2],"Ogre Global");
			
		if(MainOgre.verboso){
			System.out.println(arrayCamposFijos[0]+": "+path_ogre); // DEBUG
			System.out.println(arrayCamposFijos[1]+": "+path_agenda); // DEBUG
			System.out.println(arrayCamposFijos[2]+": "+xmailer); // DEBUG
			System.out.println(arrayCamposBoolean[0]+": "+jck1); // DEBUG
			System.out.println(arrayCamposBoolean[1]+": "+jck2); // DEBUG
			System.out.println(arrayCamposBoolean[2]+": "+jck3); // DEBUG
			System.out.println(arrayCamposBoolean[3]+": "+jck4); // DEBUG
			System.out.println(arrayCamposBoolean[4]+": "+jck5); // DEBUG
			System.out.println(arrayCamposBoolean[5]+": "+jck6); // DEBUG
			System.out.println(arrayCamposString[0]+": "+jcb1); // DEBUG
			System.out.println(arrayCamposString[1]+": "+jcb2); // DEBUG
			System.out.println(arrayCamposString[2]+": "+lang); // DEBUG
		}
		
		// Cerramos el fichero.
		fis.close();
		
		establecer_locales();
	}

	public static void establecer_locales() {
		localeActual = new Locale(lang);
		/* El primer modo es preferible por su organizacin, pero el mtodo getClass()
		 * no se porta bien con atributos y mtodos estticos. */ 
		textos = ResourceBundle.getBundle("TextBundle", localeActual);
		//String recurso = new MailProperties().getClass().getResource("/lang/TextBundle").toString();
		//System.out.println(recurso);
		//textos = ResourceBundle.getBundle(recurso, localeActual);
		
		MainOgre.mensajeErrorEstandard = System.getProperty("line.separator") + System.getProperty("line.separator") +
		MailProperties.textos.getString("mensaje_error_estandard") +
		System.getProperty("line.separator");
	}

}
