package base;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.Properties;
import javax.swing.JOptionPane;

/******************************************************************************
 * 																			  *
 *  Copyright 2004 Jordi Oliveras & Marcos Valerio.							  *							  
 * 																			  *
 *  This file is part of Ogre.												  *
 *																			  *
 *  Ogre is free software; you can redistribute it and/or modify			  *
 *  it under the terms of the GNU General Public License as published by	  *
 *  the Free Software Foundation; either version 2 of the License, or		  *
 *  (at your option) any later version.										  *
 *																			  *
 *  Ogre is distributed in the hope that it will be useful,					  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of			  *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the			  *
 *  GNU General Public License for more details.							  *
 *                       													  *
 *  You should have received a copy of the GNU General Public License         *
 *  along with Ogre; if not, write to the Free Software                       *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 * 																			  *
 ******************************************************************************
 * 																			  *
 * MainOgre - Clase que se encarga de algunas tareas de inicializacin de la  *
 * aplicacin.																  *
 * 																			  *
 ******************************************************************************/

public class MainOgre {
	
	public static String mensajeErrorEstandard;
								   
	/* Cabe resear que estos atributos son STATIC FINAL porque
	 * de este modo, la compilacin a bytecodes no tendr en cuenta
	 * los rangos de cdigo de DEBUG, omitindolos, y eliminando as
	 * la afectacin de rendimiento de la aplicacin. */
	public final static boolean verboso = false; // Queremos DEBUG?

	/* ---Mtodo activar_log_errores--- */
	/* Redirige la salida estndard de error a un fichero de log. */
	public static void activar_log_errores(){
		try {
			PrintStream salidaError = new PrintStream(new BufferedOutputStream(new FileOutputStream(System.getProperty("user.home")+"/"+"Ogre.error", true)));
			System.setErr(salidaError);
		} catch(Exception e){
			JOptionPane.showMessageDialog(null, MailProperties.textos.getString("fallo_log_errores")+mensajeErrorEstandard, MailProperties.textos.getString("error"), JOptionPane.ERROR_MESSAGE);
			e.getMessage();
			e.printStackTrace();
		}
	}

	/* ---Metodo establecer_config--- */
	/* Se asegura que haya un fichero de configuracion. */
	public static void establecer_config() throws IOException{
		
		File file = new File(System.getProperty("user.home")+"/.Ogre.properties");
		
		if(!file.exists()){
			FileOutputStream fos = new FileOutputStream(System.getProperty("user.home")+"/.Ogre.properties");
			MailProperties.escribir_properties_default();
			fos.close();
			MailProperties.leer_properties();
		} else {
			MailProperties.leer_properties();
		}
	}

	/* ---Metodo establecer_mailbox_y_agenda--- */
	/* Se asegura que haya una estructura mailbox y agenda creadas. */
	public static void establecer_mailbox_y_agenda() throws IOException{
		
		//String estructura_dirs[] = {"in", "out", "queued", "draft", "trash"};
		/* Entrantes, Salientes, Encolados(p.ej.no pudieron enviarse), Borradores y Basura */
		
		Properties properties = new Properties();
		FileInputStream fis = new FileInputStream(System.getProperty("user.home")+"/.Ogre.properties");
			
		// Carga el fichero de properties.
		properties.load(fis);
		
		boolean estado;
		//String ogre = properties.getProperty("path_ogre");
		//String mailbox = properties.getProperty("path_mailbox", "Ogre Global");
		String agenda = properties.getProperty("path_agenda", "Ogre Global");
		
		// Comprobacion estructura ogre...
		File file = new File(MailProperties.path_ogre);
		if(!file.exists()){
			estado = file.mkdir();
			if(!estado){
				if(MainOgre.verboso){
					System.out.println("No se cre "+MailProperties.path_ogre+"; algo fue mal!"); // DEBUG
					System.out.println("Cuestin de permisos?Path incorrecto?"); // DEBUG
					System.out.println("Saliendo!!!"); // DEBUG
					System.exit(0);
				}
			}
		}
						
		// Comprobacion estructura agenda
		file = new File(agenda);
		if(!file.exists()){
			estado = file.mkdir();
			if(!estado){
				if(MainOgre.verboso){
					System.out.println("No se cre el agenda; algo fue mal!"); // DEBUG
					System.out.println("Cuestin de permisos?Path incorrecto?"); // DEBUG
					System.out.println("Saliendo!!!"); // DEBUG
					System.exit(0);
				}
			}
		}
			
	}
		
}
