package gui;

import javax.swing.ImageIcon;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.text.Position;
import base.CuentaMail;
import base.MailProperties;

/******************************************************************************
 * 																			  *
 *  Copyright 2004 Jordi Oliveras & Marcos Valerio.							  *
 * 																			  *
 *  This file is part of Ogre.												  *
 *																			  *
 *  Ogre is free software; you can redistribute it and/or modify			  *
 *  it under the terms of the GNU General Public License as published by	  *
 *  the Free Software Foundation; either version 2 of the License, or		  *
 *  (at your option) any later version.										  *
 *																			  *
 *  Ogre is distributed in the hope that it will be useful,					  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of			  *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the			  *
 *  GNU General Public License for more details.							  *
 *                       													  *
 *  You should have received a copy of the GNU General Public License         *
 *  along with Ogre; if not, write to the Free Software                       *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 * 																			  *
 ****************************************************************************** 
 * 																			  *
 * ArbolCuentas - Clase que representa el rbol de cuentas y subdirectorios   *
 * que alberga cada una de ellas.											  *
 * 																			  *
 ******************************************************************************/

public final class ArbolCuentas {

	private JTree tree;
	private DefaultTreeModel treeModel;
	OgreMailTableModel omtm;
	DefaultMutableTreeNode topNode = new DefaultMutableTreeNode(MailProperties.textos.getString("nodo_cuentas")); // Creamos un nodo padre.
	String directoriosCuenta[] = {"inbox", "outbox", "draft", "queued", "trash"};
	boolean borrarCuenta = false;

	/* Constructor */
	public ArbolCuentas(CuentaMail cuentaMail, OgreMailTableModel omtm){
		
		int x=0;
		treeModel = new DefaultTreeModel(topNode);
		DefaultMutableTreeNode cuenta, directorio;
		
		this.omtm = omtm;
		
		tree = new JTree(treeModel);
		tree.setEditable(false); // El JTree no ser editable/modificable.
		tree.setShowsRootHandles(true); // Se mostrarn los cambios en el rbol cuando se hagan.
		tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION); // Establecemos seleccion simple.
		
		// Render para los iconos.
		DefaultTreeCellRenderer renderer = new DefaultTreeCellRenderer();
		renderer.setLeafIcon(new ImageIcon(getClass().getResource("/images/middle.png")));
		tree.setCellRenderer(renderer);
		
		String[] cuentas = cuentaMail.descubrir_cuentas();
		
		// Crea los nodos de las cuentas.
		while(x<cuentas.length){
			
			// Construimos el nombre real de la cuenta.
			String item = cuentas[x].replaceAll("_config", "");
			String unItem = item.replace('.', ' ');
			String otroItem = unItem.trim();
			// O bien:
			// String item = cuentas[x].replaceAll("_config", "");
			// String unItem = (item.replace('.', ' ')).trim();
			//System.out.println("otroItem: "+otroItem); // DEBUG
			 
			cuenta = new DefaultMutableTreeNode(otroItem);
			aadirNodo(cuenta, topNode);
				
			x++;
		}
	}
	
	/* ---Mtodo getTree--- */
	/* Devuelve el rbol actual. */
	JTree getTree(){
		return tree;
	}
	
	/* ---Mtodo borrarNodo--- */
	/* Borra un nodo del modelo del arbol. */
	void borrarNodo(DefaultMutableTreeNode nodoAborrar){

		/* Este booleano indica a CuentasTreeSelectionListener que se est borrando
		 * una cuenta, y por ello, cuando le mande informacin de posicin a la clase
		 * OgreMailTableModel, le indicar que la tabla esta vaca y, por tanto, no hay
		 * nada en su interior (porque de hecho, se borr la cuenta y todos sus contenidos
		 *  y ya no exixte). Si NO se hace as, cuando la cuenta es borrada y la tabla
		 * trata de actualizar sus contenidos, no encuentra la informacin de esa cuenta. 
		 * Hay que indicar que sto slo es para el caso en que la cuenta borrada est
		 * expandida, o con algun subdirectorio seleccionado o mostrando algn correo. */		
		borrarCuenta = true;
		
		int startRow = 0;
		String prefix = nodoAborrar.toString();
		TreePath treePath = tree.getNextMatch(prefix, startRow, Position.Bias.Forward);
		MutableTreeNode node = (MutableTreeNode) treePath.getLastPathComponent();
		
		// Borrar nodo; si tiene descendientes, tambin se borran.
		treeModel.removeNodeFromParent(node);
		
		borrarCuenta = false;
	}
	
	/* ---Mtodo aadirNodo--- */
	/* Aade un nodo al modelo del arbol. */
	void aadirNodo(DefaultMutableTreeNode nodoAaadir, DefaultMutableTreeNode nodoPadre){
		
		// Primero aadimos el nodo que representa la Cuenta...
		treeModel.insertNodeInto(nodoAaadir, nodoPadre, nodoPadre.getChildCount());
		
		// ...Y luego los directorios del mismo.
		for(int i=0; i<directoriosCuenta.length; i++)
			treeModel.insertNodeInto(new DefaultMutableTreeNode(directoriosCuenta[i]), nodoAaadir, nodoAaadir.getChildCount());
	}
	
	/* ---Mtodo seleccionNodo--- */
	void seleccionNodo(DefaultMutableTreeNode nodoASeleccionar, DefaultMutableTreeNode nodoPadre){
		
		int startRow = 0;
		String prefix = nodoPadre.toString();
		TreePath treePath = tree.getNextMatch(prefix, startRow, Position.Bias.Forward);
		tree.expandPath(treePath);
		
		startRow = 0;
		prefix = nodoASeleccionar.toString();
		treePath = tree.getNextMatch(prefix, startRow, Position.Bias.Forward);
		tree.setSelectionPath(treePath);
	}

}
