package gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.tree.DefaultMutableTreeNode;
import base.CuentaMail;
import base.MailProperties;
import base.MainOgre;

/******************************************************************************
 * 																			  *
 *  Copyright 2004 Jordi Oliveras & Marcos Valerio.							  *
 * 																			  *
 *  This file is part of Ogre.												  *
 *																			  *
 *  Ogre is free software; you can redistribute it and/or modify			  *
 *  it under the terms of the GNU General Public License as published by	  *
 *  the Free Software Foundation; either version 2 of the License, or		  *
 *  (at your option) any later version.										  *
 *																			  *
 *  Ogre is distributed in the hope that it will be useful,					  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of			  *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the			  *
 *  GNU General Public License for more details.							  *
 *                       													  *
 *  You should have received a copy of the GNU General Public License         *
 *  along with Ogre; if not, write to the Free Software                       *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 * 																			  *
 ****************************************************************************** 
 * 																			  *
 * OgreCreacionCuenta - Clase que representa el dilogo que asiste al usuario *
 * para la creacin de una nueva cuenta. Actualmente es muy parco y en un     *
 * futuro se prev extenderlo ms, ofreciendo ms soporte al proceso de       *
 * creacin de una cuenta.													  *
 * 																			  *
 ******************************************************************************/

public class OgreCreacionCuenta extends JFrame{
	
	CuentaMail cuentaMailNueva;
	ArbolCuentas arbolCuentas;
	Container contentPane = getContentPane();
	GridBagLayout gridbag = new GridBagLayout();
	GridBagConstraints constraints = new GridBagConstraints();
	JPanel panel = new JPanel();
	JPanel panel2 = new JPanel(gridbag);
	JButton cancelar, siguiente;
	BotonActionListener botonListener = new BotonActionListener();
	
	public OgreCreacionCuenta(CuentaMail cuentaMailNueva, ArbolCuentas arbolCuentas) {
		
		// Gestion L&F (multi-arquitectura).
		try {
			UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
		} catch(Exception e){
			JOptionPane.showMessageDialog(null, e.getMessage()+MainOgre.mensajeErrorEstandard, MailProperties.textos.getString("error"), JOptionPane.ERROR_MESSAGE);
			e.getMessage();
			e.printStackTrace();
		}

		this.cuentaMailNueva = cuentaMailNueva;
		this.arbolCuentas = arbolCuentas;
		
		super.setTitle(MailProperties.textos.getString("titulo_asistente_creacion"));
		super.setResizable(false);

		JLabel label = new JLabel(MailProperties.textos.getString("texto_asistente_inicial"));

		// Colocamos el Logo.
		JLabel banner = new JLabel(new ImageIcon(getClass().getResource("/images/OgreBanner.png")));
		banner.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		contentPane.add(banner, BorderLayout.CENTER);
	
		panel2.setBorder(BorderFactory.createEmptyBorder(0,10,0,5));
		
		// Colocamos la Bienvenida.
		constraints.insets = new Insets(15,3,0,5);
		constraints.gridx = 0;
		constraints.gridy = 2;
		constraints.gridwidth = 1;
		constraints.fill = GridBagConstraints.HORIZONTAL;
		gridbag.setConstraints(label, constraints);
		panel2.add(label);
		contentPane.add(panel2,BorderLayout.EAST);
		
		// Colocamos los botones de Cancelar y Siguiente.	
		panel.setLayout(new FlowLayout());
		cancelar = new JButton(MailProperties.textos.getString("boton_asistente_cancelar"));
		cancelar.addActionListener(botonListener);
		panel.add(cancelar);
		siguiente = new JButton(MailProperties.textos.getString("boton_asistente_siguiente"));
		siguiente.addActionListener(botonListener);
		panel.add(siguiente);
		contentPane.add(panel,BorderLayout.SOUTH);
		
		/* 
		 * Desde que NO usamos un contenedor de alto nivel para hacer de content pane
		 * (como un JPanel), si queremos poder tocar los bordes, debemos hacer un cast
		 * (a un JComponent) de lo que nos devuelve el frame al pedirle su content pane.
		 * De esta forma ya podemos operar con los bordes.
		 * */
		JComponent contentPaneCast = (JComponent) getContentPane();
		contentPaneCast.setBorder(BorderFactory.createEmptyBorder(
		5, //top
		5, //left
		0, //bottom
		5) //right
		);
		
		// Posicionamiento ventana.
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension frameSize = getPreferredSize();
		setLocation(screenSize.width/2 - (frameSize.width/2), screenSize.height/2 - (frameSize.height/2));

		// Listener para el cerrado de ventana.
		addWindowListener(new WindowAdapter(){
			public void windowClosing(WindowEvent e){
				// Liberamos la ventana.
				setVisible(false);
				dispose();
			}
		});
		
		pack();
		setVisible(true);
	}
	
	/* BotonesListener - Oyente de los clics de los botones. */
	private class BotonActionListener implements ActionListener {

		JLabel labelNombreCuenta, labelEmail, labelNombreUsuario, labelContrasea, labelServidorPop, labelServidorSmtp;
		JTextField fieldNombreCuenta, fieldEmail, fieldNombreUsuario, fieldServidorPop, fieldServidorSmtp;
		JPasswordField fieldContrasea;

		public void actionPerformed(ActionEvent e) {
		
			if(((JButton) e.getSource()).getText().equals(MailProperties.textos.getString("boton_asistente_siguiente"))){	
				
				// Redibujamos el panel dos para introducir los datos.
				panel2.removeAll();
					
				JLabel titulo = new JLabel(MailProperties.textos.getString("texto_asistente_creando"));
			 	String cadenaTexto = MailProperties.textos.getString("texto_asistente_creando2");
										 
			   	JTextArea areaTexto = new JTextArea(cadenaTexto);
			   	areaTexto.setBackground(getBackground());
			   	areaTexto.setLineWrap(true);
			   	areaTexto.setEditable(false);
				 
			   	labelNombreCuenta = new JLabel(MailProperties.textos.getString("etiqueta_asistente_nombrecuenta"));
			   	fieldNombreCuenta = new JTextField();
			   	fieldNombreCuenta.setPreferredSize(new Dimension(200,20));
			   	labelEmail = new JLabel(MailProperties.textos.getString("etiqueta_asistente_email"));
			   	fieldEmail = new JTextField();
			   	fieldEmail.setPreferredSize(new Dimension(200,20));
			   	labelNombreUsuario = new JLabel(MailProperties.textos.getString("etiqueta_asistente_nombreusuario"));
			   	fieldNombreUsuario = new JTextField();
			   	fieldNombreUsuario.setPreferredSize(new Dimension(200,20));
			   	labelContrasea = new JLabel(MailProperties.textos.getString("etiqueta_asistente_contrasea"));
			   	fieldContrasea = new JPasswordField();
			   	fieldContrasea.setPreferredSize(new Dimension(200,20));
			   	labelServidorPop = new JLabel(MailProperties.textos.getString("etiqueta_asistente_servidorpop"));
			   	fieldServidorPop = new JTextField();
			   	fieldServidorPop.setPreferredSize(new Dimension(200,20));
			   	labelServidorSmtp = new JLabel(MailProperties.textos.getString("etiqueta_asistente_servidorsmtp"));
			   	fieldServidorSmtp = new JTextField();
			   	fieldServidorSmtp.setPreferredSize(new Dimension(200,20));
		   		panel2.setBorder(BorderFactory.createEmptyBorder(0,10,0,5));	
				
		   		// Ttulo.
		   		constraints.gridx = 0;
		  		constraints.gridy = 0;
		   		constraints.gridwidth = 2;
		   		constraints.fill = GridBagConstraints.HORIZONTAL;
		   		gridbag.setConstraints(titulo, constraints);
		   		panel2.add(titulo);
			
		   		// LabelTexto.
		   		constraints.insets = new Insets(10,0,0,0);
				constraints.gridx = 0;
		   		constraints.gridy = 1;
		   		constraints.gridwidth = 2;
		   		constraints.fill = GridBagConstraints.HORIZONTAL;
		   		gridbag.setConstraints(areaTexto, constraints);
		   		panel2.add(areaTexto);
			
		   		// NombreCuenta.
		   		constraints.insets = new Insets(25,0,0,0);
		   		constraints.gridx = 0;
		   		constraints.gridy = 2;
		   		constraints.gridwidth = 1;
		   		constraints.fill = GridBagConstraints.HORIZONTAL;
		   		gridbag.setConstraints(labelNombreCuenta, constraints);
		   		panel2.add(labelNombreCuenta);
			
		   		constraints.gridx = 1;
		   		constraints.gridy = 2;
		   		constraints.gridwidth = GridBagConstraints.REMAINDER; // El componente es el ltimo de la fila.
		   		gridbag.setConstraints(fieldNombreCuenta, constraints);
		   		panel2.add(fieldNombreCuenta);
			
		   		// EMail.
		   		constraints.insets = new Insets(0,0,0,0);
		   		constraints.gridx = 0;
		   		constraints.gridy = 3;
		   		constraints.gridwidth = 1;
		   		gridbag.setConstraints(labelEmail, constraints);
		   		panel2.add(labelEmail);
			
		   		constraints.gridx = 1;
		   		constraints.gridy = 3;
		   		constraints.gridwidth = GridBagConstraints.REMAINDER;
		   		gridbag.setConstraints(fieldEmail, constraints);
		   		panel2.add(fieldEmail);
			
		   		// NombreUsuario.
		   		constraints.insets = new Insets(20,0,0,0); // Separador de componentes (delimitador del espacio de cada uno).
		   		constraints.gridx = 0;
		   		constraints.gridy = 4;
		   		constraints.gridwidth = 1;
		   		gridbag.setConstraints(labelNombreUsuario, constraints);
		   		panel2.add(labelNombreUsuario);
			
		   		constraints.gridx = 1;
		   		constraints.gridy = 4;
		   		constraints.gridwidth = GridBagConstraints.REMAINDER;
		   		gridbag.setConstraints(fieldNombreUsuario, constraints);
		   		panel2.add(fieldNombreUsuario);
			
		   		// Contrasea.
		   		constraints.insets = new Insets(0,0,0,0);
				constraints.gridx = 0;
		   		constraints.gridy = 5;
		   		constraints.gridwidth = 1;
		   		gridbag.setConstraints(labelContrasea, constraints);
		   		panel2.add(labelContrasea);
		
		   		constraints.gridx = 1;
		   		constraints.gridy = 5;
		   		constraints.gridwidth = GridBagConstraints.REMAINDER;
		   		gridbag.setConstraints(fieldContrasea, constraints);
		   		panel2.add(fieldContrasea);
			
		   		// ServidorPOP.
		   		constraints.insets = new Insets(20,0,0,0);
		   		constraints.gridx = 0;
		   		constraints.gridy = 6;
		   		constraints.gridwidth = 1;
		   		gridbag.setConstraints(labelServidorPop, constraints);
		   		panel2.add(labelServidorPop);
			
		   		constraints.gridx = 1;
		   		constraints.gridy = 6;
		   		constraints.gridwidth = GridBagConstraints.REMAINDER;
		   		gridbag.setConstraints(fieldServidorPop, constraints);
		   		panel2.add(fieldServidorPop);
			
		   		// Servidor SMTP.
		   		constraints.insets = new Insets(0,0,0,0);
			    constraints.gridx = 0;
		   		constraints.gridy = 7;
		   		constraints.gridwidth = 1;
		   		gridbag.setConstraints(labelServidorSmtp, constraints);
		   		panel2.add(labelServidorSmtp);
			
		   		constraints.gridx = 1;
		   		constraints.gridy = 7;
		   		constraints.gridwidth = GridBagConstraints.REMAINDER;
	 		    gridbag.setConstraints(fieldServidorSmtp, constraints);
		   		panel2.add(fieldServidorSmtp);
				panel2.repaint();
		   		
		   		// Redibujamos el panel para colocar los nuevos botones.
		   		panel.removeAll();
		   		
			   	// Colocamos los botones de Cancelar y Finalizar.
			   	panel.setLayout(new FlowLayout());
			   	cancelar = new JButton(MailProperties.textos.getString("boton_asistente_cancelar"));
			   	cancelar.addActionListener(botonListener);
			   	panel.add(cancelar);
			   	siguiente = new JButton(MailProperties.textos.getString("boton_asistente_finalizar"));
			   	siguiente.addActionListener(botonListener);
			   	panel.add(siguiente);
			   	panel.repaint();
			   	
			   	contentPane.add(panel,BorderLayout.SOUTH);   
			   	contentPane.add(panel2, BorderLayout.EAST);
			   		
			   	setVisible(true);

			} else if(((JButton) e.getSource()).getText().equals(MailProperties.textos.getString("boton_asistente_finalizar"))){
		
				try {
		
					String nombreCuenta = fieldNombreCuenta.getText();
					if(nombreCuenta.equalsIgnoreCase("")) throw new Exception();
					cuentaMailNueva.nombreCuenta = fieldNombreCuenta.getText();
					cuentaMailNueva.email = fieldEmail.getText();
					cuentaMailNueva.nombreUsuario = fieldNombreUsuario.getText();
					cuentaMailNueva.contrasea = fieldContrasea.getText();
					cuentaMailNueva.hostPop = fieldServidorPop.getText();
					cuentaMailNueva.hostSmtp = fieldServidorSmtp.getText();
					
					try {
						boolean resultado = cuentaMailNueva.crear_cuenta();
						if(!resultado){
							JOptionPane.showMessageDialog(null, MailProperties.textos.getString("fallo_crear_cuenta"), MailProperties.textos.getString("aviso"), JOptionPane.INFORMATION_MESSAGE);
						} else {
							setVisible(false);
							dispose();
							arbolCuentas.aadirNodo(new DefaultMutableTreeNode(cuentaMailNueva.nombreCuenta), arbolCuentas.topNode);
							OgreCuenta.listarCuentas(); // Actualizamos la lista de OgreCuenta.
						}
					} catch (IOException e1) {
						JOptionPane.showMessageDialog(null, MailProperties.textos.getString("fallo_crear_cuenta2")+MainOgre.mensajeErrorEstandard, MailProperties.textos.getString("error"), JOptionPane.ERROR_MESSAGE);
						e1.getMessage();
						e1.printStackTrace();
					}
					
				} catch(Exception exc){
					JOptionPane.showMessageDialog(null, MailProperties.textos.getString("fallo_nombre_cuenta"), MailProperties.textos.getString("aviso"), JOptionPane.WARNING_MESSAGE);
					//exc.getMessage();
					//exc.printStackTrace();
				}
				
			} else {
				setVisible(false);
				dispose();	
			}
		}
	}

}
