package gui;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.UIManager;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import base.CuentaMail;
import base.MailProperties;
import base.MainOgre;

/******************************************************************************
 * 																			  *
 *  Copyright 2004 Jordi Oliveras & Marcos Valerio.							  *
 * 																			  *
 *  This file is part of Ogre.												  *
 *																			  *
 *  Ogre is free software; you can redistribute it and/or modify			  *
 *  it under the terms of the GNU General Public License as published by	  *
 *  the Free Software Foundation; either version 2 of the License, or		  *
 *  (at your option) any later version.										  *
 *																			  *
 *  Ogre is distributed in the hope that it will be useful,					  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of			  *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the			  *
 *  GNU General Public License for more details.							  *
 *                       													  *
 *  You should have received a copy of the GNU General Public License         *
 *  along with Ogre; if not, write to the Free Software                       *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 *																			  *
 ******************************************************************************
 *																			  *
 * OgreCuenta - Esta clase se encarga de representar grficamente aquellas    *
 * cuentas configuradas por el usuario en Ogre. Con esta representacin, el   *
 * usuario puede decidir qu operacin realizar sobre ellas.                  *
 *																			  *
 ******************************************************************************/

public final class OgreCuenta extends JFrame {

	JList jlistCuentas;
	static DefaultListModel listModel;
	JScrollPane jscrollpane1;
	JButton buttonCrear, buttonEditar, buttonEliminar;
	JPanel panel = null;
	static CuentaMail cuentaMail;
	ArbolCuentas arbolCuentas;
	static String[] cuentas;
	static String[] cuentasMostrables;
	int cuentaSeleccionada;
	boolean haySeleccion = false;

	public OgreCuenta(CuentaMail aCuentaMail, ArbolCuentas aArbolCuentas){
		
		super(MailProperties.textos.getString("titulo_cuentas_ventana"));
		super.setResizable(false);
		
		cuentaMail = aCuentaMail;
		arbolCuentas = aArbolCuentas;
		
		// Gestion L&F (multi-arquitectura).
		try {
			UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
		} catch(Exception e){
			JOptionPane.showMessageDialog(null, e.getMessage()+MainOgre.mensajeErrorEstandard, MailProperties.textos.getString("error"), JOptionPane.ERROR_MESSAGE);
			e.getMessage();
			e.printStackTrace();
		}
		
		// Cogemos el Layout del Panel de Contenido del Frame, que es BorderLayout por defecto.
		Container contentPane = getContentPane();
		
		/* Ahora aadiremos primero un JList de cuentas que ir a la izquierda,
		 * y la aadiremos al Frame. */
		listModel = new DefaultListModel();
		jlistCuentas = new JList(listModel); //cuentasMostrables
		listarCuentas(); // Aadir las cuentas a la lista (concretamente al modelo de la lista).
		jlistCuentas.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		//jlistCuentas.setSelectedIndex(0);
		jlistCuentas.addListSelectionListener(new SeleccionarCuentaListener());
		
		jscrollpane1 = new JScrollPane(jlistCuentas);
		jscrollpane1.setPreferredSize(new Dimension(100,100)); // Tamao default.
		contentPane.add(jscrollpane1, BorderLayout.CENTER); 
		
		/* Ahora un panel que contendr 3 botones en un BoxLayout. */
		panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		BotonesListener bl = new BotonesListener();
		buttonCrear = new JButton(MailProperties.textos.getString("boton_cuentas_nueva"));
		buttonCrear.addActionListener(bl);
		buttonEditar = new JButton(MailProperties.textos.getString("boton_cuentas_editar"));
		buttonEditar.addActionListener(bl);
		buttonEliminar = new JButton(MailProperties.textos.getString("boton_cuentas_eliminar"));
		buttonEliminar.addActionListener(bl);
		
		// Desactiva los botones Editar/Eliminar.
		desactivaBotones();
		
		panel.add(buttonCrear);
		panel.add(buttonEditar);
		panel.add(buttonEliminar);
		
		panel.setBorder(BorderFactory.createEmptyBorder(15,20,15,20));
		contentPane.add(panel, BorderLayout.SOUTH);

		/* 
		 * Desde que NO usamos un contenedor de alto nivel para hacer de content pane
		 * (como un JPanel), si queremos poder tocar los bordes, debemos hacer un cast
		 * (a un JComponent) de lo que nos devuelve el frame al pedirle su content pane.
		 * De esta forma ya podemos operar con los bordes.
		 * */
		JComponent contentPaneCast = (JComponent) getContentPane();
		contentPaneCast.setBorder(BorderFactory.createEmptyBorder(
		5, //top
		5, //left
		0, //bottom
		5) //right
		);

		// Posicionamiento ventana.
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension frameSize = getPreferredSize();
		setLocation(screenSize.width/2 - (frameSize.width/2), screenSize.height/2 - (frameSize.height/2));

		// Listener para el cerrado de ventana.
		addWindowListener(new WindowAdapter(){
			public void windowClosing(WindowEvent e){
				// Liberamos la ventana.
				setVisible(false);
				dispose();
			}
		});

		pack();
		setVisible(true);
	}
	
	///////////////
	// LISTENERS //
	///////////////
	
	/* SeleccionarCuentaListener - Oyente de seleccin de la jlist de cuentas. */
	private class SeleccionarCuentaListener implements ListSelectionListener {
		
		public void valueChanged(ListSelectionEvent e){
			
			if(e.getValueIsAdjusting())
				return;
	
			JList lista = (JList) e.getSource();
			if(!lista.isSelectionEmpty()) {
				activaBotones();
				cuentaSeleccionada = lista.getSelectedIndex();
				haySeleccion = true;
				if(MainOgre.verboso) System.out.println("Indice "+cuentaSeleccionada+" seleccionado."); // DEBUG
				if(MainOgre.verboso) System.out.println("cuentaSeleccionada: "+cuentaSeleccionada);
			}
		}
	}
	
	/* BotonesListener - Oyente de los clics de los botones. */
	private class BotonesListener implements ActionListener {

		public void actionPerformed(ActionEvent e){
			JButton boton = (JButton) e.getSource();
			
			if(boton.getText().equalsIgnoreCase(MailProperties.textos.getString("boton_cuentas_nueva"))){
				if(MainOgre.verboso) System.out.println("Nueva"); // DEBUG
				// Llamar al wizard.
				CuentaMail cuentaMailNueva = new CuentaMail();
				OgreCreacionCuenta occ = new OgreCreacionCuenta(cuentaMailNueva, arbolCuentas);
				desactivaBotones();
				
			} else if(boton.getText().equalsIgnoreCase(MailProperties.textos.getString("boton_cuentas_editar"))){
				
				if(haySeleccion == true){
				
					if(MainOgre.verboso) System.out.println("Editar"); // DEBUG
					// Abrir una ventana con la info de la cuenta seleccionada en el paso anterior.
					CuentaMail cuentaMailEditable = new CuentaMail();
					try {
						cuentaMailEditable.seleccionar_cuenta(cuentasMostrables[cuentaSeleccionada]);
					} catch (IOException e1) {
						JOptionPane.showMessageDialog(null, MailProperties.textos.getString("fallo_editar_cuenta")+MainOgre.mensajeErrorEstandard, MailProperties.textos.getString("error"), JOptionPane.ERROR_MESSAGE);
						e1.getMessage();
						e1.printStackTrace();
					} catch (ClassNotFoundException e1) {
						JOptionPane.showMessageDialog(null, MailProperties.textos.getString("fallo_editar_cuenta")+MainOgre.mensajeErrorEstandard, MailProperties.textos.getString("error"), JOptionPane.ERROR_MESSAGE);
						e1.getMessage();
						e1.printStackTrace();
					}
					//System.out.println("cuentas[cuentaSeleccionada]: "+cuentas[cuentaSeleccionada]); // DEBUG
					//System.out.println("cuentasMostrables[cuentaSeleccionada]: "+cuentasMostrables[cuentaSeleccionada]); // DEBUG
					OgreEdicionCuenta oec = new OgreEdicionCuenta(cuentaMail, cuentaMailEditable);
					desactivaBotones();
					
				} else {
					JOptionPane.showMessageDialog(null, "Seleccione antes la cuenta a editar.", "Atencin", JOptionPane.WARNING_MESSAGE);
				}
				
			} else if(boton.getText().equalsIgnoreCase(MailProperties.textos.getString("boton_cuentas_eliminar"))){
				
				if((cuentas.length > 0)&&(haySeleccion == true)){
				
					if(MainOgre.verboso) System.out.println("Eliminar"); // DEBUG
					// Simplemente eliminar la cuenta.
					if(MainOgre.verboso) System.out.println("Borrando cuenta: "+cuentasMostrables[cuentaSeleccionada]);
					CuentaMail cuentaMailBorrable = new CuentaMail();
					cuentaMailBorrable.eliminar_cuenta(cuentasMostrables[cuentaSeleccionada]);
					if(MainOgre.verboso) System.out.println("cuentaSeleccionada: "+cuentaSeleccionada);
					
					arbolCuentas.seleccionNodo(new DefaultMutableTreeNode(cuentasMostrables[cuentaSeleccionada]), arbolCuentas.topNode);
					arbolCuentas.borrarNodo(new DefaultMutableTreeNode(cuentasMostrables[cuentaSeleccionada]));
					
					listarCuentas();
					haySeleccion = false;
					desactivaBotones();
					
				} else {
					JOptionPane.showMessageDialog(null, "Seleccione antes la cuenta a eliminar.", "Atencin", JOptionPane.WARNING_MESSAGE);
				}
			}
		}
	}

	/////////////
	// MT0D0S //
	/////////////
	
	/* ---Mtodo listarCuentas--- */
	/* Obtiene un listado de las Cuentas existentes. */
	public static void listarCuentas(){
		listModel.clear();
		
		cuentas = cuentaMail.descubrir_cuentas();
		cuentasMostrables = new String[cuentas.length];
		for(int i=0; i<cuentas.length; i++){
			// Cogemos el nombre real de cada cuenta.
			String item = cuentas[i].replaceAll("_config", "");
			String unItem = item.replace('.', ' ');
			String otroItem = unItem.trim();
			//cuentas[i] = otroItem;
			//cuentasMostrables.add(i, otroItem);
			cuentasMostrables[i] = new String(otroItem);
			listModel.addElement(cuentasMostrables[i]);
		}
		if(MainOgre.verboso) System.out.println("cuentas.length: "+cuentas.length);
	}
	
	/* ---Mtodo desactivaBotones--- */
	/* Activa/Desactiva los botones de Editar/Eliminar Cuenta, segn sea necesario. */
	private void activaBotones(){ 
		buttonEditar.setEnabled(true);
		buttonEliminar.setEnabled(true);
	}
	
	/* ---Mtodo desactivaBotones--- */
	/* Activa/Desactiva los botones de Editar/Eliminar Cuenta, segn sea necesario. */
	private void desactivaBotones(){ 
		buttonEditar.setEnabled(false);
		buttonEliminar.setEnabled(false);
	}
	
}
