package gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.UIManager;
import base.CuentaMail;
import base.MailProperties;
import base.MainOgre;

/******************************************************************************
 * 																			  *
 *  Copyright 2004 Jordi Oliveras & Marcos Valerio.							  *
 * 																			  *
 *  This file is part of Ogre.												  *
 *																			  *
 *  Ogre is free software; you can redistribute it and/or modify			  *
 *  it under the terms of the GNU General Public License as published by	  *
 *  the Free Software Foundation; either version 2 of the License, or		  *
 *  (at your option) any later version.										  *
 *																			  *
 *  Ogre is distributed in the hope that it will be useful,					  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of			  *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the			  *
 *  GNU General Public License for more details.							  *
 *                       													  *
 *  You should have received a copy of the GNU General Public License         *
 *  along with Ogre; if not, write to the Free Software                       *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 * 																			  *
 ****************************************************************************** 
 *                                                                            *
 * OgreEdicionCuenta - Clase que representa el dilogo que asiste al usuario  *
 * para la edicin de una cuenta existente.									  *
 * 																			  *
 ******************************************************************************/

public final class OgreEdicionCuenta extends JFrame {	

	CuentaMail cuentaMail, cuentaMailEditable;
	JTextField fieldNombreCuenta, fieldEmail, fieldNombreUsuario, fieldServidorPop, fieldServidorSmtp;
	JPasswordField fieldContrasea;

	public OgreEdicionCuenta(CuentaMail cuentaMail, CuentaMail cuentaMailEditable){
		
		super(MailProperties.textos.getString("titulo_asistente_edicion"));
		super.setResizable(false);
		
		this.cuentaMail = cuentaMail;
		this.cuentaMailEditable = cuentaMailEditable;
		
		// Gestion L&F (multi-arquitectura).
		try {
			UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
		} catch(Exception e){
			JOptionPane.showMessageDialog(null, e.getMessage()+MainOgre.mensajeErrorEstandard, MailProperties.textos.getString("error"), JOptionPane.ERROR_MESSAGE);
			e.getMessage();
			e.printStackTrace();
		}
		
		// Cogemos el Layout del Panel de Contenido del Frame, que es BorderLayout por defecto.
		Container contentPane = getContentPane();

		/* 1) Creamos una label con el banner y lo aadimos en el centro. */
		JLabel banner = new JLabel(new ImageIcon(getClass().getResource("/images/OgreBanner.png")));
		banner.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		contentPane.add(banner, BorderLayout.CENTER);
		
		/* 2) Creamos un conjunto de labels y textfields, y los introduciremos en un panel. 
		 * Este panel ser aadido a la derecha del BorderLayout. */
		 JLabel titulo = new JLabel(MailProperties.textos.getString("texto_asistente_editando"));
		 
		 String cadenaTexto = MailProperties.textos.getString("texto_asistente_editando2");
		 
		JTextArea areaTexto = new JTextArea(cadenaTexto);
		areaTexto.setBackground(this.getBackground());
		areaTexto.setLineWrap(true);
		areaTexto.setEditable(false);
		 
		JLabel labelNombreCuenta = new JLabel(MailProperties.textos.getString("etiqueta_asistente_nombrecuenta"));
		fieldNombreCuenta = new JTextField(cuentaMailEditable.nombreCuenta);
		fieldNombreCuenta.setPreferredSize(new Dimension(200,20));
		fieldNombreCuenta.setEditable(false); // El nombre no se podr cambiar.
		JLabel labelEmail = new JLabel(MailProperties.textos.getString("etiqueta_asistente_email"));
		fieldEmail = new JTextField(cuentaMailEditable.email);
		fieldEmail.setPreferredSize(new Dimension(200,20));
		JLabel labelNombreUsuario = new JLabel(MailProperties.textos.getString("etiqueta_asistente_nombreusuario"));
		fieldNombreUsuario = new JTextField(cuentaMailEditable.nombreUsuario);
		fieldNombreUsuario.setPreferredSize(new Dimension(200,20));
		JLabel labelContrasea = new JLabel(MailProperties.textos.getString("etiqueta_asistente_contrasea"));
		fieldContrasea = new JPasswordField(cuentaMailEditable.contrasea);
		fieldContrasea.setPreferredSize(new Dimension(200,20));
		JLabel labelServidorPop = new JLabel(MailProperties.textos.getString("etiqueta_asistente_servidorpop"));
		fieldServidorPop = new JTextField(cuentaMailEditable.hostPop);
		fieldServidorPop.setPreferredSize(new Dimension(200,20));
		JLabel labelServidorSmtp = new JLabel(MailProperties.textos.getString("etiqueta_asistente_servidorsmtp"));
		fieldServidorSmtp = new JTextField(cuentaMailEditable.hostSmtp);
		fieldServidorSmtp.setPreferredSize(new Dimension(200,20));
		
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints constraints = new GridBagConstraints();
		JPanel panel = new JPanel(gridbag);
		panel.setBorder(BorderFactory.createEmptyBorder(0,10,0,5));	
		
		// Ttulo.
		constraints.gridx = 0;
		constraints.gridy = 0;
		constraints.gridwidth = 2;
		constraints.fill = GridBagConstraints.HORIZONTAL;
		gridbag.setConstraints(titulo, constraints);
		panel.add(titulo);
		
		// LabelTexto.
		constraints.insets = new Insets(10,0,0,0);
		constraints.gridx = 0;
		constraints.gridy = 1;
		constraints.gridwidth = 2;
		constraints.fill = GridBagConstraints.HORIZONTAL;
		gridbag.setConstraints(areaTexto, constraints);
		panel.add(areaTexto);
		
		// NombreCuenta.
		constraints.insets = new Insets(25,0,0,0);
		constraints.gridx = 0;
		constraints.gridy = 2;
		constraints.gridwidth = 1;
		constraints.fill = GridBagConstraints.HORIZONTAL;
		gridbag.setConstraints(labelNombreCuenta, constraints);
		panel.add(labelNombreCuenta);
		
		constraints.gridx = 1;
		constraints.gridy = 2;
		constraints.gridwidth = GridBagConstraints.REMAINDER; // El componente es el ltimo de la fila.
		gridbag.setConstraints(fieldNombreCuenta, constraints);
		panel.add(fieldNombreCuenta);
		
		// EMail.
		constraints.insets = new Insets(0,0,0,0);
		constraints.gridx = 0;
		constraints.gridy = 3;
		constraints.gridwidth = 1;
		gridbag.setConstraints(labelEmail, constraints);
		panel.add(labelEmail);
		
		constraints.gridx = 1;
		constraints.gridy = 3;
		constraints.gridwidth = GridBagConstraints.REMAINDER;
		gridbag.setConstraints(fieldEmail, constraints);
		panel.add(fieldEmail);
		
		// NombreUsuario.
		constraints.insets = new Insets(20,0,0,0); // Separador de componentes (delimitador del espacio de cada uno).
		constraints.gridx = 0;
		constraints.gridy = 4;
		constraints.gridwidth = 1;
		gridbag.setConstraints(labelNombreUsuario, constraints);
		panel.add(labelNombreUsuario);
		
		constraints.gridx = 1;
		constraints.gridy = 4;
		constraints.gridwidth = GridBagConstraints.REMAINDER;
		gridbag.setConstraints(fieldNombreUsuario, constraints);
		panel.add(fieldNombreUsuario);
		
		// Contrasea.
		constraints.insets = new Insets(0,0,0,0);
		constraints.gridx = 0;
		constraints.gridy = 5;
		constraints.gridwidth = 1;
		gridbag.setConstraints(labelContrasea, constraints);
		panel.add(labelContrasea);
		
		constraints.gridx = 1;
		constraints.gridy = 5;
		constraints.gridwidth = GridBagConstraints.REMAINDER;
		gridbag.setConstraints(fieldContrasea, constraints);
		panel.add(fieldContrasea);
		
		// ServidorPOP.
		constraints.insets = new Insets(20,0,0,0);
		constraints.gridx = 0;
		constraints.gridy = 6;
		constraints.gridwidth = 1;
		gridbag.setConstraints(labelServidorPop, constraints);
		panel.add(labelServidorPop);
		
		constraints.gridx = 1;
		constraints.gridy = 6;
		constraints.gridwidth = GridBagConstraints.REMAINDER;
		gridbag.setConstraints(fieldServidorPop, constraints);
		panel.add(fieldServidorPop);
		
		// Servidor SMTP.
		constraints.insets = new Insets(0,0,0,0);
		constraints.gridx = 0;
		constraints.gridy = 7;
		constraints.gridwidth = 1;
		gridbag.setConstraints(labelServidorSmtp, constraints);
		panel.add(labelServidorSmtp);
		
		constraints.gridx = 1;
		constraints.gridy = 7;
		constraints.gridwidth = GridBagConstraints.REMAINDER;
		gridbag.setConstraints(fieldServidorSmtp, constraints);
		panel.add(fieldServidorSmtp);
		
		contentPane.add(panel, BorderLayout.EAST);
		
		/* 3) Creamos otro panel con un FlowLayout de botones. */
		BotonesListener bl = new BotonesListener();
		JButton botonCancelar = new JButton(MailProperties.textos.getString("boton_asistente_cancelar"));
		botonCancelar.addActionListener(bl);
		JButton botonAceptar = new JButton(MailProperties.textos.getString("boton_asistente_aceptar"));
		botonAceptar.addActionListener(bl);
		JPanel panel2 = new JPanel(new FlowLayout());
		panel2.setBorder(BorderFactory.createEmptyBorder(10,0,0,0));
		panel2.add(botonCancelar);
		panel2.add(botonAceptar);
		contentPane.add(panel2, BorderLayout.SOUTH);
		
		/* 
		 * Desde que NO usamos un contenedor de alto nivel para hacer de content pane
		 * (como un JPanel), si queremos poder tocar los bordes, debemos hacer un cast
		 * (a un JComponent) de lo que nos devuelve el frame al pedirle su content pane.
		 * De esta forma ya podemos operar con los bordes.
		 * */
		JComponent contentPaneCast = (JComponent) getContentPane();
		contentPaneCast.setBorder(BorderFactory.createEmptyBorder(
		5, //top
		5, //left
		5, //bottom
		5) //right
		);
		
		// Posicionamiento ventana.
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension frameSize = getPreferredSize();
		setLocation(screenSize.width/2 - (frameSize.width/2), screenSize.height/2 - (frameSize.height/2));

		// Listener para el cerrado de ventana.
		addWindowListener(new WindowAdapter(){
			public void windowClosing(WindowEvent e){
				// Liberamos la ventana.
				setVisible(false);
				dispose();
			}
		});

		pack();
		setVisible(true);
	}
	
	///////////////
	// LISTENERS //
	///////////////
	
	/* BotonesListener - Oyente de los clics de los botones. */
	private class BotonesListener implements ActionListener {

		public void actionPerformed(ActionEvent e){
			JButton boton = (JButton) e.getSource();
			if(boton.getText().equalsIgnoreCase(MailProperties.textos.getString("boton_asistente_aceptar"))){
				/* Modificamos los datos de la cuenta seleccionada, cerramos esta
				 * ventana y deberamos actualizar el JList del panel anterior). */
				cuentaMailEditable.nombreCuenta = fieldNombreCuenta.getText();
				cuentaMailEditable.email = fieldEmail.getText();
				cuentaMailEditable.nombreUsuario = fieldNombreUsuario.getText();
				cuentaMailEditable.contrasea = fieldContrasea.getText();
				cuentaMailEditable.hostPop = fieldServidorPop.getText();
				cuentaMailEditable.hostSmtp = fieldServidorSmtp.getText();
				try {
					cuentaMailEditable.modificar_cuenta();
					// Si la cuenta a editar es la actualmente seleccionada...
					if((cuentaMail.nombreCuenta != null)&&(cuentaMail.nombreCuenta.equalsIgnoreCase(cuentaMailEditable.nombreCuenta))){
						//cuentaMail = cuentaMailEditable;
						cuentaMail.seleccionar_cuenta(cuentaMailEditable.nombreCuenta);
					}
				} catch (IOException e1) {
					JOptionPane.showMessageDialog(null, MailProperties.textos.getString("fallo_editar_cuenta")+MainOgre.mensajeErrorEstandard, MailProperties.textos.getString("error"), JOptionPane.ERROR_MESSAGE);
					e1.getMessage();
					e1.printStackTrace();
				} catch (ClassNotFoundException e2) {
					JOptionPane.showMessageDialog(null, MailProperties.textos.getString("fallo_editar_cuenta")+MainOgre.mensajeErrorEstandard, MailProperties.textos.getString("error"), JOptionPane.ERROR_MESSAGE);
					e2.getMessage();
					e2.printStackTrace();
				}
				
				// Liberamos la ventana.
				setVisible(false);
				dispose();
				OgreCuenta.listarCuentas(); // Actualizamos la lista de OgreCuenta.
				
			} else {
				// nicamente cerramos esta ventana.
				setVisible(false);
				dispose();
			}
		}
	}

}
