package gui;

/******************************************************************************
 * 																			  *
 *  Copyright 2004 Jordi Oliveras & Marcos Valerio. 						  *
 * 																			  *
 *  This file is part of Ogre.												  *
 *																			  *
 *  Ogre is free software; you can redistribute it and/or modify			  *
 *  it under the terms of the GNU General Public License as published by	  *
 *  the Free Software Foundation; either version 2 of the License, or		  *
 *  (at your option) any later version.										  *
 *																			  *
 *  Ogre is distributed in the hope that it will be useful,					  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of			  *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the			  *
 *  GNU General Public License for more details.							  *
 *                       													  *
 *  You should have received a copy of the GNU General Public License         *
 *  along with Ogre; if not, write to the Free Software                       *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 * 																			  *
 ******************************************************************************
 *																			  *
 * OgreMailTableModel - Esta clase representa nuestro modelo de datos de la   * 
 * tabla del interfaz principal para construir y tratar los datos de los      *
 * correos que se mostrarn al usuario.  									  *  
 * 																			  *
 ******************************************************************************/

import java.io.FileInputStream;
import java.util.Date;
import javax.mail.Address;
import javax.mail.internet.MimeMessage;
import javax.swing.JOptionPane;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;
import base.CuentaMail;
import base.GestorLocalMail;
import base.MailProperties;
import base.MainOgre;

public final class OgreMailTableModel extends AbstractTableModel {
	
	private String nodo;
	private CuentaMail cuentaMail;
	String mensajes[];
	 
	public Object[][] filas; // Definicin del array (matriz).
		
	public String[] columnas = {"",
			  			 MailProperties.textos.getString("cabecera_asunto"), 
						 MailProperties.textos.getString("cabecera_de"),
						 MailProperties.textos.getString("cabecera_fecha")};

	/* Constructor */
	public OgreMailTableModel() { 
		this.addTableModelListener(new MiTableModelListener());
	}

	/////////////////
	// MT0D0S SET //
	/////////////////

	/* ---Mtodo setNodo--- */
	/* Setea un nodo al valor del nodo del JTree seleccionado. */
	public void setNodo(String nombreNodo){
		nodo = nombreNodo;
	}
	
//	/* ---Mtodo setMailProperties--- */
//	public void setMailProperties(MailProperties aMailProperties){
//		mailProperties = aMailProperties;
//	}
	
	/* ---Mtodo setCuentaMail--- */
	public void setCuentaMail(CuentaMail aCuentaMail){
		cuentaMail = aCuentaMail;
	}
		
	///////////////
	// LISTENERS //
	///////////////

	/* ---MiTableModelListener - Oyente de seleccion de la tabla--- */
	/* Vigila los cambios de datos de la tabla de mails. */
	//	Listener para cambios en la tabla (en su modelo, para ser exactos). //
	private class MiTableModelListener implements TableModelListener { 
		public void tableChanged(TableModelEvent e) {

			int i = 0, y = 0, num_mensajes;

			try {

				if(!nodo.equalsIgnoreCase("TABLA_VACIA")){ // !TABLA_VACIA
					
					// Obtiene los nombres de los ficheros del directorio seleccionado.
					mensajes = GestorLocalMail.recuperar_mail_local(cuentaMail, nodo);
	
					/* Reserva espacio para la matriz de datos. La ltima columna es 
				 	* extra para almacenar el nombre del fichero en disco que representa 
				 	* a la fila. */
					filas = new Object[mensajes.length][columnas.length+1]; // Array (matriz) de referencias.
															   	// ! //
					num_mensajes = mensajes.length;															
															   
				} else { // TABLA_VACIA
					
					num_mensajes = 0;
					filas = new Object[num_mensajes][columnas.length+1]; // Array (matriz) de referencias.
				}
				
				// Para cada uno de los mensajes del directorio...
				for(int x=0; x<num_mensajes; x++){
					
					// ... abrimos el fichero en el que se encuentra, cogemos los campos
					// del mensaje que nos interesan, y los colocamos en la matriz que
					// mostraremos en el TableModel. 
					
					if(nodo == null) // NEW
						nodo = "inbox"; // NEW
					
					// Nos puede servir esto para construir messages para trabajar localmente?:
					// (La session nos importa un comino ahora mismo... al carajo con ella)
					MimeMessage message = new MimeMessage(null, new FileInputStream(MailProperties.path_ogre+cuentaMail.nombreCuenta+"/"+nodo+"/"+mensajes[x]));
					
					
					filas[x][y] = new Boolean(false);
					y++;
					
					try {
						String subject = message.getSubject();
						filas[x][y] = subject;
					} catch(NullPointerException npe) {
						filas[x][y] = "___Sin asunto___";
					} catch(Exception exc){
						filas[x][y] = "___Asunto invlido___";
					}
					y++;
					
					try {
						String address = ((Address) message.getFrom() [0]).toString();
						filas[x][y] = address;
					} catch(NullPointerException npe){
						filas[x][y] = "___Sin remite___"; 
					} catch(Exception exc){
						filas[x][y] = "___Remite invlido___";
					}
					y++;
					
					try {
						Date date = message.getSentDate();
						filas[x][y] = date;
					} catch(NullPointerException npe){
						filas[x][y] = "___Sin fecha___";
					} catch(Exception exc){
						filas[x][y] = "___Fecha invlida___";
					}
					y++;
					
					filas[x][y] = mensajes[x]; // El nombre del fichero en disco!
					
					y=0;
				}
			
			} catch(Exception exc){
				JOptionPane.showMessageDialog(null, MailProperties.textos.getString("fallo_en_tabla")+MainOgre.mensajeErrorEstandard, MailProperties.textos.getString("error"), JOptionPane.ERROR_MESSAGE);
				exc.getMessage();
				exc.printStackTrace();
			}

		}
	}

	/////////////////////////////////////////////////
	// MET0D0S implementados DE AbstractTableModel //
	/////////////////////////////////////////////////
	
	public int getColumnCount() {
		return columnas.length;
	}
        
	public int getRowCount() {
		if (filas!=null) // Si no se comprueba esto y no hay datos... PAM!!! xDDD
			return filas.length;
		else
			return 0;
	}

	public String getColumnName(int col) {
		return columnas[col];
	}

	public Object getValueAt(int row, int col) {
		return filas[row][col];
	}
	
	/* JTable usa este metodo para determinar el renderer/editor
	 * por defecto para cada celda. Su implementacin pretende interpretar
	 * los tipos de datos contenidos en la tabla correctamente. */
	public Class getColumnClass(int c) {
		return getValueAt(0, c).getClass();
	}
	
	/*
	 * No se necesita implementar a menos que la tabla sea editable.
	 * nicamente el checkbox es editable, lo dems no.
	 */
	public boolean isCellEditable(int row, int col) {
		// NOTA: El dato/celda es constante, independientemente
		// de donde aparezca la celda en la pantalla.
		if (col > 0) 
			return false;
		else
			return true;
	}
	
	/*
	 * No se necesita implementar a menos que los datos contenidos en
	 * la tabla puedan cambiar.
	 */
	public void setValueAt(Object value, int row, int col) {
		filas[row][col] = value;
	}

}
